 precision highp float;
 varying mediump vec2 textureCoordinate;
 
 uniform sampler2D inputImageTexture;
 uniform sampler2D inputImageTexture2;
 
 uniform float inputImageTextureHeight;
 uniform float inputImageTextureWidth;
 
 float texture2Size = 1024.0;
 
 uniform float strength;

 // gray
 float NCGray(vec4 textcolor)
{
    float gray = 0.2125 * textcolor.r + 0.7154 * textcolor.g + 0.0721 * textcolor.b;
    
    return gray;
}
 
 // tone mapping
 vec4 NCTonemapping(vec4 textcolor)
{
    
    vec4 mapped;
    mapped.r = texture2D(inputImageTexture2, vec2(textcolor.r, 0.0)).r;
    mapped.g = texture2D(inputImageTexture2, vec2(textcolor.g, 0.0)).g;
    mapped.b = texture2D(inputImageTexture2, vec2(textcolor.b, 0.0)).b;
    mapped.a = textcolor.a;
    
    return mapped;
}
 
 // textcolor control
 vec4 NCColorControl(vec4 textcolor, float saturation, float brightness, float contrast)
{
    float gray = NCGray(textcolor);
    
    textcolor.rgb = vec3(saturation) * textcolor.rgb + vec3(1.0-saturation) * vec3(gray);
    textcolor.r = clamp(textcolor.r, 0.0, 1.0);
    textcolor.g = clamp(textcolor.g, 0.0, 1.0);
    textcolor.b = clamp(textcolor.b, 0.0, 1.0);
    
    textcolor.rgb = vec3(contrast) * (textcolor.rgb - vec3(0.5)) + vec3(0.5);
    textcolor.r = clamp(textcolor.r, 0.0, 1.0);
    textcolor.g = clamp(textcolor.g, 0.0, 1.0);
    textcolor.b = clamp(textcolor.b, 0.0, 1.0);
    
    textcolor.rgb = textcolor.rgb + vec3(brightness);
    textcolor.r = clamp(textcolor.r, 0.0, 1.0);
    textcolor.g = clamp(textcolor.g, 0.0, 1.0);
    textcolor.b = clamp(textcolor.b, 0.0, 1.0);
    
    return textcolor;
}
 
 // hue adjust
 vec4 NCHueAdjust(vec4 textcolor, float hueAdjust)
{
    vec3 kRGBToYPrime = vec3(0.299, 0.587, 0.114);
    vec3 kRGBToI = vec3(0.595716, -0.274453, -0.321263);
    vec3 kRGBToQ = vec3(0.211456, -0.522591, 0.31135);
    
    vec3 kYIQToR   = vec3(1.0, 0.9563, 0.6210);
    vec3 kYIQToG   = vec3(1.0, -0.2721, -0.6474);
    vec3 kYIQToB   = vec3(1.0, -1.1070, 1.7046);
    
    float yPrime = dot(textcolor.rgb, kRGBToYPrime);
    float I = dot(textcolor.rgb, kRGBToI);
    float Q = dot(textcolor.rgb, kRGBToQ);
    
    float hue = atan(Q, I);
    float chroma  = sqrt (I * I + Q * Q);
    
    hue -= hueAdjust;
    
    Q = chroma * sin (hue);
    I = chroma * cos (hue);
    
    textcolor.r = dot(vec3(yPrime, I, Q), kYIQToR);
    textcolor.g = dot(vec3(yPrime, I, Q), kYIQToG);
    textcolor.b = dot(vec3(yPrime, I, Q), kYIQToB);
    
    return textcolor;
}
 
 // colorMatrix
 vec4 NCColorMatrix(vec4 textcolor, float red, float green, float blue, float alpha, vec4 bias)
{
    textcolor = textcolor * vec4(red, green, blue, alpha) + bias;
    
    return textcolor;
}
 
 // multiply blend
 vec4 NCMultiplyBlend(vec4 overlay, vec4 base)
{
    vec4 outputColor;
    
    float a = overlay.a + base.a * (1.0 - overlay.a);
    
    //    // normal blend
    //    outputColor.r = (base.r * base.a + overlay.r * overlay.a * (1.0 - base.a))/a;
    //    outputColor.g = (base.g * base.a + overlay.g * overlay.a * (1.0 - base.a))/a;
    //    outputColor.b = (base.b * base.a + overlay.b * overlay.a * (1.0 - base.a))/a;
    
    
    // multiply blend
    outputColor.rgb = ((1.0-base.a) * overlay.rgb * overlay.a + (1.0-overlay.a) * base.rgb * base.a + overlay.a * base.a * overlay.rgb * base.rgb) / a;
    
    
    outputColor.a = a;
    
    return outputColor;
}
 
 // xy should be a integer position (e.g. pixel position on the screen)
 // similar to a texture lookup but is only ALU
 float PseudoRandom(vec2 co)
{
    //    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
    mediump float a = 12.9898;
    mediump float b = 78.233;
    mediump float c = 43758.5453;
    mediump float dt= dot(co.xy ,vec2(a,b));
    mediump float sn= mod(dt,3.14);
    return fract(sin(sn) * c);
}
 
 void main()
{
    vec4 originColor = texture2D(inputImageTexture, textureCoordinate);
    vec4 textcolor = texture2D(inputImageTexture, textureCoordinate);
    
    textcolor.a = 1.0;
    
    // textcolor control
//    textcolor = NCColorControl(textcolor, 0.6, -0.06, 0.75);
    textcolor = NCColorControl(textcolor, 0.5, 0.1, 0.9);
    
    // rand
	  float x = textureCoordinate.x*inputImageTextureWidth/texture2Size;
      float y = textureCoordinate.y*inputImageTextureHeight/texture2Size;

      vec4 rd = texture2D(inputImageTexture2, vec2( fract(x), fract(y)));
//    vec4 rd = texture2D(inputImageTexture2, textureCoordinate);
//    float rand_number1 = PseudoRandom(textureCoordinate.xy);
//    float rand_number2 = PseudoRandom(textureCoordinate.yx);
//    float rand_number3 = PseudoRandom(vec2(rand_number1, rand_number2));
//    float rand_number4 = PseudoRandom(vec2(rand_number2, rand_number1));
//    float rand_number5 = PseudoRandom(vec2(rand_number3, rand_number4));
    
//    vec4 rd = vec4(rand_number1, rand_number3, rand_number5, 1.0);
    
//    if(rand_number4>0.2)
//        rd = vec4(1.0);
    
    // rand textcolor control
//    rd = NCColorControl(rd, 0.65, 0.1, 0.7);
    rd = NCColorControl(rd, 1.0, 0.4, 1.2);
    
    // normal blend
//    rd.a *= 1.0;
    textcolor = NCMultiplyBlend(rd, textcolor);
    
    // textcolor matrix
//    textcolor = NCColorMatrix(textcolor, 1.0, 1.0, 1.0, 1.0, vec4(-0.1, -0.1, -0.1, 0));
    textcolor = NCColorMatrix(textcolor, 1.0, 1.0, 1.0, 1.0, vec4(-0.15, -0.15, -0.15, 0));
    
    textcolor.rgb = mix(originColor.rgb, textcolor.rgb, strength);
    gl_FragColor = textcolor;
}